@@ -1,8 +1,8 @@
 # -*- encoding: utf-8 -*-
 
 __author__ = "Open Observatory of Network Interference"
-__version__ = "1.0.2"
+__version__ = "1.1.0"
 
 __all__ = ['config', 'inputunit', 'kit',
-           'lib', 'nettest', 'oonicli', 'reporter',
+           'lib', 'nettest', 'oonicli', 'report', 'reporter',
            'templates', 'utils']
