@@ -1,16 +1,15 @@
 import os
 import re
 import time
+import sys
 from hashlib import sha256
 
-from twisted.internet import defer, reactor
+from twisted.internet import defer
 from twisted.trial.runner import filenameToModule
 from twisted.python import usage, reflect
 
-from ooni import geoip
 from ooni.tasks import Measurement
-from ooni.utils import log, checkForRoot
-from ooni import otime
+from ooni.utils import log, checkForRoot, sanitize_options
 from ooni.settings import config
 
 from ooni import errors as e
@@ -18,25 +17,10 @@ from ooni import errors as e
 from inspect import getmembers
 from StringIO import StringIO
 
+
 class NoTestCasesFound(Exception):
     pass
 
-def get_test_methods(item, method_prefix="test_"):
-    """
-    Look for test_ methods in subclasses of NetTestCase
-    """
-    test_cases = []
-    try:
-        assert issubclass(item, NetTestCase)
-        methods = reflect.prefixedMethodNames(item, method_prefix)
-        test_methods = []
-        for method in methods:
-            test_methods.append(method_prefix + method)
-        if test_methods:
-            test_cases.append((item, test_methods))
-    except (TypeError, AssertionError):
-        pass
-    return test_cases
 
 def getTestClassFromFile(net_test_file):
     """
@@ -53,10 +37,12 @@ def getTestClassFromFile(net_test_file):
         except (TypeError, AssertionError):
             pass
 
+
 def getOption(opt_parameter, required_options, type='text'):
     """
     Arguments:
-        usage_options: a list as should be the optParameters of an UsageOptions class.
+        usage_options: a list as should be the optParameters of an UsageOptions
+            class.
 
         required_options: a list containing the strings of the options that are
             required.
@@ -79,16 +65,19 @@ def getOption(opt_parameter, required_options, type='text'):
         required = False
 
     return {'description': description,
-        'value': default, 'required': required,
-        'type': type
-    }
+            'value': default, 'required': required,
+            'type': type
+            }
+
 
 def getArguments(test_class):
     arguments = {}
     if test_class.inputFile:
         option_name = test_class.inputFile[0]
-        arguments[option_name] = getOption(test_class.inputFile,
-                test_class.requiredOptions, type='file')
+        arguments[option_name] = getOption(
+            test_class.inputFile,
+            test_class.requiredOptions,
+            type='file')
     try:
         list(test_class.usageOptions.optParameters)
     except AttributeError:
@@ -96,16 +85,20 @@ def getArguments(test_class):
 
     for opt_parameter in test_class.usageOptions.optParameters:
         option_name = opt_parameter[0]
-        opt_type="text"
+        opt_type = "text"
         if opt_parameter[3].lower().startswith("file"):
-            opt_type="file"
-        arguments[option_name] = getOption(opt_parameter,
-                test_class.requiredOptions, type=opt_type)
+            opt_type = "file"
+        arguments[option_name] = getOption(
+            opt_parameter,
+            test_class.requiredOptions,
+            type=opt_type)
 
     return arguments
 
+
 def test_class_name_to_name(test_class_name):
-    return test_class_name.lower().replace(' ','_')
+    return test_class_name.lower().replace(' ', '_')
+
 
 def getNetTestInformation(net_test_file):
     """
@@ -124,21 +117,23 @@ def getNetTestInformation(net_test_file):
 
     test_id = os.path.basename(net_test_file).replace('.py', '')
     information = {'id': test_id,
-        'name': test_class.name,
-        'description': test_class.description,
-        'version': test_class.version,
-        'arguments': getArguments(test_class),
-        'path': net_test_file,
-    }
+                   'name': test_class.name,
+                   'description': test_class.description,
+                   'version': test_class.version,
+                   'arguments': getArguments(test_class),
+                   'path': net_test_file,
+                   }
     return information
 
+
 class NetTestLoader(object):
     method_prefix = 'test'
     collector = None
     requiresTor = False
 
     def __init__(self, options, test_file=None, test_string=None):
-        self.onionInputRegex = re.compile("(httpo://[a-z0-9]{16}\.onion)/input/([a-z0-9]{64})$")
+        self.onionInputRegex = re.compile(
+            "(httpo://[a-z0-9]{16}\.onion)/input/([a-z0-9]{64})$")
         self.options = options
         self.testCases = []
 
@@ -205,21 +200,21 @@ class NetTestLoader(object):
         for input_file in self.inputFiles:
             input_file_hashes.append(input_file['hash'])
 
+        options = sanitize_options(self.options)
         test_details = {'start_time': time.time(),
-            'probe_asn': config.probe_ip.geodata['asn'],
-            'probe_cc': config.probe_ip.geodata['countrycode'],
-            'probe_ip': config.probe_ip.geodata['ip'],
-            'probe_city': config.probe_ip.geodata['city'],
-            'test_name': self.testName,
-            'test_version': self.testVersion,
-            'software_name': 'ooniprobe',
-            'software_version': software_version,
-            'options': self.options,
-            'input_hashes': input_file_hashes
-        }
+                        'probe_asn': config.probe_ip.geodata['asn'],
+                        'probe_cc': config.probe_ip.geodata['countrycode'],
+                        'probe_ip': config.probe_ip.geodata['ip'],
+                        'probe_city': config.probe_ip.geodata['city'],
+                        'test_name': self.testName,
+                        'test_version': self.testVersion,
+                        'software_name': 'ooniprobe',
+                        'software_version': software_version,
+                        'options': options,
+                        'input_hashes': input_file_hashes
+                        }
         return test_details
 
-
     def _parseNetTestOptions(self, klass):
         """
         Helper method to assemble the options into a single UsageOptions object
@@ -293,7 +288,7 @@ class NetTestLoader(object):
 
         if not test_cases:
             raise e.NoTestCasesFound
-        
+
         self.setupTestCases(test_cases)
 
     def setupTestCases(self, test_cases):
@@ -331,7 +326,11 @@ class NetTestLoader(object):
         """
         for klass in self.testClasses:
             options = self.usageOptions()
-            options.parseOptions(self.options)
+            try:
+                options.parseOptions(self.options)
+            except usage.UsageError:
+                tb = sys.exc_info()[2]
+                raise e.OONIUsageError(self), None, tb
 
             if options:
                 klass.localOptions = options
@@ -362,7 +361,9 @@ class NetTestLoader(object):
             pass
         return test_cases
 
+
 class NetTestState(object):
+
     def __init__(self, allTasksDone):
         """
         This keeps track of the state of a running NetTests case.
@@ -409,6 +410,7 @@ class NetTestState(object):
         self.completedScheduling = True
         self.checkAllTasksDone()
 
+
 class NetTest(object):
     director = None
 
@@ -435,10 +437,10 @@ class NetTest(object):
         self.done.addCallback(self.doneNetTest)
 
         self.state = NetTestState(self.done)
-    
+
     def __str__(self):
         return ' '.join(tc.name for tc, _ in self.testCases)
-    
+
     def doneNetTest(self, result):
         if not self.summary:
             return
@@ -458,9 +460,6 @@ class NetTest(object):
         """
         self.state.taskDone()
 
-        if len(self.report.reporters) == 0:
-            raise e.AllReportersFailed
-
         return report_results
 
     def makeMeasurement(self, test_instance, test_method, test_input=None):
@@ -485,15 +484,17 @@ class NetTest(object):
 
         if self.director:
             measurement.done.addCallback(self.director.measurementSucceeded,
-                    measurement)
+                                         measurement)
             measurement.done.addErrback(self.director.measurementFailed,
-                    measurement)
+                                        measurement)
         return measurement
 
     @defer.inlineCallbacks
     def initializeInputProcessor(self):
         for test_class, _ in self.testCases:
-            test_class.inputs = yield defer.maybeDeferred(test_class().getInputProcessor)
+            test_class.inputs = yield defer.maybeDeferred(
+                test_class().getInputProcessor
+            )
             if not test_class.inputs:
                 test_class.inputs = [None]
 
@@ -511,7 +512,10 @@ class NetTest(object):
                 test_instance.summary = self.summary
                 for method in test_methods:
                     log.debug("Running %s %s" % (test_class, method))
-                    measurement = self.makeMeasurement(test_instance, method, input)
+                    measurement = self.makeMeasurement(
+                        test_instance,
+                        method,
+                        input)
                     measurements.append(measurement.done)
                     self.state.taskCreated()
                     yield measurement
@@ -524,6 +528,7 @@ class NetTest(object):
                     # Call the postProcessor, which must return a single report
                     # or a deferred
                     post.addCallback(test_instance.postProcessor)
+
                     def noPostProcessor(failure, report):
                         failure.trap(e.NoPostProcessor)
                         return report
@@ -531,30 +536,32 @@ class NetTest(object):
                     post.addCallback(self.report.write)
 
                 if self.report and self.director:
-                    #ghetto hax to keep NetTestState counts are accurate
+                    # ghetto hax to keep NetTestState counts are accurate
                     [post.addBoth(self.doneReport) for _ in measurements]
 
         self.state.allTasksScheduled()
 
+
 class NetTestCase(object):
+
     """
     This is the base of the OONI nettest universe. When you write a nettest
     you will subclass this object.
 
     * inputs: can be set to a static set of inputs. All the tests (the methods
-      starting with the "test" prefix) will be run once per input.  At every run
-      the _input_ attribute of the TestCase instance will be set to the value of
-      the current iteration over inputs.  Any python iterable object can be set
-      to inputs.
+      starting with the "test" prefix) will be run once per input. At every
+      run the _input_ attribute of the TestCase instance will be set to the
+      value of the current iteration over inputs.  Any python iterable object
+      can be set to inputs.
 
-    * inputFile: attribute should be set to an array containing the command line
-      argument that should be used as the input file. Such array looks like
-      this:
+    * inputFile: attribute should be set to an array containing the command
+      line argument that should be used as the input file. Such array looks
+      like this:
 
           ``["commandlinearg", "c", "default value" "The description"]``
 
-      The second value of such arrray is the shorthand for the command line arg.
-      The user will then be able to specify inputs to the test via:
+      The second value of such arrray is the shorthand for the command line
+      arg. The user will then be able to specify inputs to the test via:
 
           ``ooniprobe mytest.py --commandlinearg path/to/file.txt``
 
@@ -578,12 +585,14 @@ class NetTestCase(object):
 
     * requiresRoot: set to True if the test must be run as root.
 
-    * usageOptions: a subclass of twisted.python.usage.Options for processing of command line arguments
+    * usageOptions: a subclass of twisted.python.usage.Options for processing
+        of command line arguments
 
     * localOptions: contains the parsed command line arguments.
 
     Quirks:
-    Every class that is prefixed with test *must* return a twisted.internet.defer.Deferred.
+    Every class that is prefixed with test *must* return a
+    twisted.internet.defer.Deferred.
     """
     name = "This test is nameless"
     author = "Jane Doe <foo@example.com>"
@@ -601,20 +610,21 @@ class NetTestCase(object):
     optParameters = None
     baseParameters = None
     baseFlags = None
-    
+
     requiredTestHelpers = {}
     requiredOptions = []
     requiresRoot = False
     requiresTor = False
 
     localOptions = {}
+
     def _setUp(self):
         """
         This is the internal setup method to be overwritten by templates.
         """
         self.report = {}
         self.inputs = None
-    
+
     def requirements(self):
         """
         Place in here logic that will be executed before the test is to be run.
@@ -709,7 +719,7 @@ class NetTestCase(object):
 
         We check to see if it's possible to have an input file and if the user
         has specified such file.
-            
+
 
         If the operations to be done here are network related or blocking, they
         should be wrapped in a deferred. That is the return value of this
@@ -725,7 +735,7 @@ class NetTestCase(object):
 
         if self.inputs:
             return self.inputs
- 
+
         return None
 
     def _checkValidOptions(self):
@@ -739,10 +749,10 @@ class NetTestCase(object):
         for required_option in self.requiredOptions:
             log.debug("Checking if %s is present" % required_option)
             if required_option not in self.localOptions or \
-                self.localOptions[required_option] == None:
-                    missing_options.append(required_option)
+                    self.localOptions[required_option] is None:
+                missing_options.append(required_option)
         if missing_options:
-            raise e.MissingRequiredOption(missing_options)
+            raise e.MissingRequiredOption(missing_options, self)
 
     def __repr__(self):
         return "<%s inputs=%s>" % (self.__class__, self.inputs)
