@@ -1,29 +1,34 @@
 # -*- encoding: utf-8 -*-
 import random
-import string
-import subprocess
 from distutils.spawn import find_executable
 
 from twisted.python import usage
-from twisted.internet import defer, reactor, error
+from twisted.internet import reactor, error
 
 import txtorcon
 
+from ooni.settings import config
 from ooni.utils import log, onion
-from ooni import nettest
+from ooni import nettest, errors
+
+
+class TorIsNotInstalled(Exception):
+    pass
 
-class TorIsNotInstalled(Exception): pass
 
 class UsageOptions(usage.Options):
-    optParameters = [['timeout', 't', 120,
-                      'Specify the timeout after which to consider the Tor bootstrapping process to have failed'],
-                    ]
+    optParameters = [
+        ['timeout', 't', 120,
+         'Specify the timeout after which to consider '
+         'the Tor bootstrapping process to have failed'], ]
+
 
 class BridgeReachability(nettest.NetTestCase):
     name = "Bridge Reachability"
-    description = "A test for checking if bridges are reachable from a given location."
+    description = "A test for checking if bridges are reachable " \
+                  "from a given location."
     author = "Arturo Filastò"
-    version = "0.1"
+    version = "0.1.1"
 
     usageOptions = UsageOptions
 
@@ -33,15 +38,23 @@ class BridgeReachability(nettest.NetTestCase):
                  'TransportType IP:ORPort (ex. obfs2 127.0.0.1:443)']
 
     requiredOptions = ['file']
-    
+
     def requirements(self):
         if not onion.find_tor_binary():
-            raise TorIsNotInstalled("For instructions on installing Tor see: https://www.torproject.org/download/download")
+            raise TorIsNotInstalled(
+                "For instructions on installing Tor see: "
+                "https://www.torproject.org/download/download")
 
     def setUp(self):
         self.tor_progress = 0
         self.timeout = int(self.localOptions['timeout'])
 
+        if self.timeout > config.advanced.measurement_timeout:
+            log.err("The measurement timeout is less than the bridge reachability test timeout")
+            log.err("Adjust your ooniprobe.conf file by setting the "
+                    "advanced: measurement_timeout: value to %d" % self.timeout)
+            raise errors.InvalidConfigFile("advanced->measurement_timeout < %d" % self.timeout)
+
         self.report['error'] = None
         self.report['success'] = None
         self.report['timeout'] = self.timeout
@@ -51,14 +64,15 @@ class BridgeReachability(nettest.NetTestCase):
         self.report['tor_progress_tag'] = None
         self.report['tor_progress_summary'] = None
         self.report['tor_log'] = None
+        self.report['obfsproxy_version'] = str(onion.obfsproxy_details['version'])
         self.report['bridge_address'] = None
 
         self.bridge = self.input
         if self.input.startswith('Bridge'):
             self.bridge = self.input.replace('Bridge ', '')
-        self.pyobfsproxy_bin = find_executable('obfsproxy')
+        self.pyobfsproxy_bin = onion.obfsproxy_details['binary']
         self.fteproxy_bin = find_executable('fteproxy')
-    
+
     def postProcessor(self, measurements):
         if 'successes' not in self.summary:
             self.summary['successes'] = []
@@ -79,6 +93,7 @@ class BridgeReachability(nettest.NetTestCase):
     def displaySummary(self, summary):
         successful_count = {}
         failure_count = {}
+
         def count(results, counter):
             for result in results:
                 if result['transport_name'] not in counter:
@@ -87,8 +102,13 @@ class BridgeReachability(nettest.NetTestCase):
         count(summary['successes'], successful_count)
         count(summary['failures'], failure_count)
 
-        working_bridges = ', '.join(["%s %s" % (x['transport_name'], x['address']) for x in summary['successes']])
-        failing_bridges = ', '.join(["%s %s (at %s%%)" % (x['transport_name'], x['address'], x['tor_progress']) for x in summary['failures']])
+        working_bridges = ', '.join(
+            ["%s %s" % (x['transport_name'], x['address'])
+             for x in summary['successes']])
+        failing_bridges = ', '.join(
+            ["%s %s (at %s%%)"
+             % (x['transport_name'], x['address'], x['tor_progress'])
+             for x in summary['failures']])
 
         log.msg("Total successes: %d" % len(summary['successes']))
         log.msg("Total failures: %d" % len(summary['failures']))
@@ -105,28 +125,42 @@ class BridgeReachability(nettest.NetTestCase):
         config = txtorcon.TorConfig()
         config.ControlPort = random.randint(2**14, 2**16)
         config.SocksPort = random.randint(2**14, 2**16)
-        log.msg("Connecting to %s with tor %s" % (self.bridge, onion.tor_details['version']))
-        
+        log.msg(
+            "Connecting to %s with tor %s" %
+            (self.bridge, onion.tor_details['version']))
+
         transport_name = onion.transport_name(self.bridge)
         if transport_name and transport_name == 'fte' and self.fteproxy_bin:
-            config.ClientTransportPlugin = "%s exec %s --managed" % (transport_name, self.fteproxy_bin)
+            config.ClientTransportPlugin = "%s exec %s --managed" % (
+                transport_name, self.fteproxy_bin)
+            log.debug("Using fte from %s" % self.fteproxy_bin)
             self.report['transport_name'] = transport_name
             self.report['bridge_address'] = self.bridge.split(' ')[1]
-        elif transport_name and transport_name == 'fte' and not self.fteproxy_bin:
+        elif transport_name and transport_name == 'fte'\
+                and not self.fteproxy_bin:
             log.err("Unable to test bridge because fteproxy is not installed")
             self.report['error'] = 'missing-fteproxy'
             return
         elif transport_name and self.pyobfsproxy_bin:
-            config.ClientTransportPlugin = "%s exec %s managed" % (transport_name, self.pyobfsproxy_bin)
+            config.ClientTransportPlugin = "%s exec %s managed" % (
+                transport_name, self.pyobfsproxy_bin)
+            if onion.OBFSProxyVersion('0.2') > onion.obfsproxy_details['version']:
+                log.err(
+                    "The obfsproxy version you are using appears to be outdated."
+                )
+                self.report['error'] = 'old-obfsproxy'
+                return
+            log.debug("Using pyobfsproxy from %s" % self.pyobfsproxy_bin)
             self.report['transport_name'] = transport_name
             self.report['bridge_address'] = self.bridge.split(' ')[1]
         elif transport_name and not self.pyobfsproxy_bin:
-            log.err("Unable to test bridge because pyobfsproxy is not installed")
+            log.err(
+                "Unable to test bridge because pyobfsproxy is not installed")
             self.report['error'] = 'missing-pyobfsproxy'
             return
         else:
             self.report['bridge_address'] = self.bridge.split(' ')[0]
-        
+
         if transport_name and transport_name == 'scramblesuit' and \
                 onion.TorVersion('0.2.5.1') > onion.tor_details['version']:
             self.report['error'] = 'unsupported-tor-version'
@@ -151,6 +185,7 @@ class BridgeReachability(nettest.NetTestCase):
 
         d = txtorcon.launch_tor(config, reactor, timeout=self.timeout,
                                 progress_updates=updates)
+
         @d.addCallback
         def setup_complete(proto):
             try:
